@@ -98,10 +98,13 @@ int  build_skc_list( STRLIST locusr, SKC_LIST *ret_skc_list,
 void set_passphrase_fd( int fd );
 int  get_passphrase_fd(void);
 DEK *passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode);
+void set_next_passphrase( const char *s );
+char *get_last_passphrase(void);
 
 /*-- getkey.c --*/
 void add_keyring( const char *name );
 const char *get_keyring( int sequence );
+const char *get_secret_keyring( int sequence );
 void add_secret_keyring( const char *name );
 int get_pubkey( PKT_public_cert *pkc, u32 *keyid );
 int get_pubkey_byname( PKT_public_cert *pkc, const char *name );
