@@ -31,8 +31,11 @@
 #include "cipher.h"
 #include "keydb.h"
 #include "main.h"
+#include "i18n.h"
 
 static int pwfd = -1;
+static char *next_pw = NULL;
+static char *last_pw = NULL;
 
 static void hash_passphrase( DEK *dek, char *pw, STRING2KEY *s2k, int create );
 
@@ -48,6 +51,34 @@ get_passphrase_fd()
     return pwfd;
 }
 
+/****************
+ * Set the passphrase to be used for the next query and only for the next
+ * one.
+ */
+void
+set_next_passphrase( const char *s )
+{
+    m_free(next_pw);
+    next_pw = NULL;
+    if( s ) {
+	next_pw = m_alloc_secure( strlen(s)+1 );
+	strcpy(next_pw, s );
+    }
+}
+
+/****************
+ * Get the last passphrase used in passphrase_to_dek.
+ * Note: This removes the passphrase from this modules and
+ * the caller must free the result.  May return NULL:
+ */
+char *
+get_last_passphrase()
+{
+    char *p = last_pw;
+    last_pw = NULL;
+    return p;
+}
+
 
 /****************
  * Get a passphrase for the secret key with KEYID, display TEXT
@@ -62,7 +93,7 @@ get_passphrase_fd()
 DEK *
 passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
 {
-    char *pw;
+    char *pw = NULL;
     DEK *dek;
     STRING2KEY help_s2k;
 
@@ -77,17 +108,32 @@ passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
 					    :DEFAULT_DIGEST_ALGO;
     }
 
-    if( keyid && !opt.batch ) {
-	char *ustr;
-	tty_printf("Need a pass phrase to unlock the secret key for:\n");
-	tty_printf("  \"" );
-	ustr = get_user_id_string( keyid );
-	tty_print_string( ustr, strlen(ustr) );
-	m_free(ustr);
-	tty_printf("\"\n\n");
+    if( keyid && !opt.batch && !next_pw ) {
+	PKT_public_cert *pkc = m_alloc_clear( sizeof *pkc );
+	size_t n;
+	char *p;
 
+	tty_printf(_("\nYou need a passphrase to unlock the secret key for\n"
+		     "user: \"") );
+	p = get_user_id( keyid, &n );
+	tty_print_string( p, n );
+	m_free(p);
+	tty_printf("\"\n");
+
+	if( !get_pubkey( pkc, keyid ) ) {
+	    const char *s = pubkey_algo_to_string( pkc->pubkey_algo );
+	    tty_printf( _("(%u-bit %s key, ID %08lX, created %s)\n"),
+		       nbits_from_pkc( pkc ), s?s:"?", (ulong)keyid[1],
+		       strtimestamp(pkc->timestamp) );
+	}
+	tty_printf("\n");
+	free_public_cert( pkc );
     }
-    if( pwfd != -1 ) { /* read the passphrase from the given descriptor */
+    if( next_pw ) {
+	pw = next_pw;
+	next_pw = NULL;
+    }
+    else if( pwfd != -1 ) { /* read the passphrase from the file */
 	int i, len;
 
 	if( !opt.batch )
@@ -130,7 +176,8 @@ passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
 	dek->keylen = 0;
     else
 	hash_passphrase( dek, pw, s2k, mode==2 );
-    m_free(pw); /* is allocated in secure memory, so it will be burned */
+    m_free(last_pw);
+    last_pw = pw;
     return dek;
 }
 
